@@ -1,5 +1,6 @@
 import logging
 import sys
+import time
 
 import requests
 from transifex.native.consts import (KEY_CHARACTER_LIMIT,
@@ -11,7 +12,9 @@ TRANSIFEX_CDS_HOST = 'https://cds.svc.transifex.net'
 TRANSIFEX_CDS_URLS = {
     'FETCH_AVAILABLE_LANGUAGES': '/languages',
     'FETCH_TRANSLATIONS_FOR_LANGUAGE': '/content/{language_code}',
-    'PUSH_SOURCE_STRINGS': '/content/'
+    'PUSH_SOURCE_STRINGS': '/content/',
+    'INVALIDATE_CACHE': '/invalidate',
+    'PURGE_CACHE': '/purge',
 }
 
 logger = logging.getLogger('transifex.native.cds')
@@ -28,6 +31,10 @@ MAPPING = {
     KEY_OCCURRENCES: 'occurrences',
 }
 
+# Number of times to retry connecting to CDS before bailing out
+MAX_RETRIES = 3
+RETRY_DELAY_SEC = 2
+
 
 class EtagStore(object):
     """ Manges etags """
@@ -73,13 +80,10 @@ class CDSHandler(object):
         languages = []
 
         try:
-            last_response_status = 202
-            while last_response_status == 202:
-                response = requests.get(
-                    self.host + cds_url,
-                    headers=self._get_headers(),
-                )
-                last_response_status = response.status_code
+            response = self.retry_get_request(
+                self.host + cds_url,
+                headers=self._get_headers(),
+            )
 
             if not response.ok:
                 logger.error(
@@ -130,16 +134,12 @@ class CDSHandler(object):
                 set(self.configured_language_codes):
 
             try:
-                last_response_status = 202
-                while last_response_status == 202:
-                    response = requests.get(
-                        (self.host +
-                         cds_url.format(language_code=language_code)),
-                        headers=self._get_headers(
-                            etag=self.etags.get(language_code)
-                        )
+                response = self.retry_get_request(
+                    (self.host + cds_url.format(language_code=language_code)),
+                    headers=self._get_headers(
+                        etag=self.etags.get(language_code)
                     )
-                    last_response_status = response.status_code
+                )
 
                 if not response.ok:
                     logger.error(
@@ -217,6 +217,38 @@ class CDSHandler(object):
 
         return response
 
+    def invalidate_cache(self, purge=False):
+        """Invalidate CDS cache.
+
+        :param bool purge: True deletes CDS cache entirely instead of
+            triggering a job to re-cache content.
+        :return: the HTTP response object
+        :rtype: requests.Response
+        """
+        if not self.secret:
+            raise Exception('You need to use `TRANSIFEX_SECRET` when '
+                            'invalidating cache')
+
+        cds_url = TRANSIFEX_CDS_URLS['PURGE_CACHE'] if purge else \
+            TRANSIFEX_CDS_URLS['INVALIDATE_CACHE']
+
+        try:
+            response = requests.post(
+                self.host + cds_url,
+                headers=self._get_headers(use_secret=True),
+                json={}
+            )
+            response.raise_for_status()
+
+        except requests.ConnectionError:
+            logger.error(
+                'Error invalidating CDS: ConnectionError')
+        except Exception as e:
+            logger.error('Error invalidating CDS: UnknownError '
+                         '(`{}`)'.format(str(e)))
+
+        return response
+
     def _serialize(self, source_string):
         """Serialize the given source string to a format suitable for the CDS.
 
@@ -259,3 +291,19 @@ class CDSHandler(object):
             headers['If-None-Match'] = etag
 
         return headers
+
+    def retry_get_request(self, *args, **kwargs):
+        """ Resilient function for GET requests """
+        retries, last_response_status = 0, 202
+        while (last_response_status == 202 or
+                500 <= last_response_status < 600 and
+                retries < MAX_RETRIES):
+
+            if 500 <= last_response_status < 600:
+                retries += 1
+                time.sleep(retries * RETRY_DELAY_SEC)
+
+            response = requests.get(*args, **kwargs)
+            last_response_status = response.status_code
+
+        return response
